#!/bin/bash

##
# $Author: john $
# $Date: 2002/07/03 15:34:20 $
# $Source: /var/src/RCS/mk-chroot.sh,v $
# $Revision: 0.2 $

# $Log: mk-chroot.sh,v $
# Revision 0.2  2002/07/03 15:34:20  john
# Rough cut.
#
##

CMD=`basename $0`

CHROOT_DIRS='bin etc usr usr/bin usr/lib lib lib/i686'
BINS='/bin/bash /bin/ls /bin/mkdir /bin/mv /bin/pwd /bin/rm'
USR_BINS='/usr/bin/id'

SH_LIBS='/lib/libnss_compat.so.2 /lib/libtermcap.so.2 /lib/libcrypto.so.2 /lib/libcrypt.so.1 /lib/libnsl.so.1 /usr/lib/libz.so.1 /usr/lib/libcrypto.so.1 /lib/ld-linux.so.2 /lib/libdl.so.2 /lib/i686/libc.so.6 /lib/libc.so.6 /lib/libcrypt.so.1 /usr/lib/libcrypto.so.1 /lib/libutil.so.1 /lib/libnsl.so.1 /usr/lib/libz.so.1'

CAT=/bin/cat
CHATTR=/usr/bin/chattr
CHMOD=/bin/chmod
CHMOD_ARGS='+x -v'
CP='/bin/cp'
CP_ARGS='-v'
ECHO=/bin/echo
GREP=/bin/grep
MKDIR=/bin/mkdir
MKDIR_ARGS='-v'

usage()
{
$CAT << _END_
Usage:

  $CMD -u newuser -h /home/newuser [-i] [-I]


  Options are:
  -d	: user home dir
  -u	: userid
  -h	: Print help.
  -i	: Lock by setting the imutable attribute on directories.
  -I	: Lock as with -i and recurse into directories.
	
_END_
exit 1
}

while getopts u:d:iIh i $*
do
  case $i in
        i ) CHATTR_ARGS='+i -V';;
	I ) CHATTR_ARGS='+i -R -V';;
	u ) CHROOT_USER=$OPTARG;;
	d ) CHROOT_HOME=$OPTARG;;
	h | \
        * ) usage ;;
  esac
done

sanity_chk() {

  if [ -z "$CHROOT_HOME" ] || [ -z "$CHROOT_USER" ]
    then usage 
  fi

  if [ -d "$CHROOT_HOME" ]
  then
    cd "$CHROOT_HOME"
  else
    echo "Home directory [$CHROOT_HOME] must exist."
  exit
  fi

  while [ "$1" ]; do
    if [ ! -e $1 ]
    then
      echo "Source file [$1] does not exist."
      exit
    fi
  shift
  done
}

create_dirs() {
  while [ "$1" ]; do
    if [ ! -d $1 ]
    then
      $MKDIR $MKDIR_ARGS $CHROOT_HOME/$1
    fi
  shift
  done
}

lockit() {
  if [ ! -z "$CHATTR_ARGS" ]
  then
    while [ "$1" ]; do
      if [ -d $1 ]
      then
        $CHATTR $CHATTR_ARGS $CHROOT_HOME/$1
      fi
    shift
    done
  fi
}

build_uid()
{
  grep /etc/passwd -e "^root" -e "^$CHROOT_USER" > $CHROOT_HOME/etc/passwd
  grep /etc/group -e "^root" -e "^$CHROOT_USER" > $CHROOT_HOME/etc/group
  echo "#!/bin/bash" > $CHROOT_HOME/usr/bin/groups
  echo "id -Gn" >> $CHROOT_HOME/usr/bin/groups
  $CHMOD $CHMOD_ARGS $CHROOT_HOME/usr/bin/groups
}

copy_files() {
    while [ "$1" ]; do
      if [ ! -e $CHROOT_HOME$1 ]
      then
        $CP $CP_ARGS $1 $CHROOT_HOME$1
      fi
    shift
    done
}

sanity_chk $BINS $USR_BINS $SH_LIBS
create_dirs $CHROOT_DIRS
copy_files $BINS
copy_files $USR_BINS
copy_files $SH_LIBS
build_uid
lockit $CHROOT_DIRS


# $Header: /var/src/RCS/mk-chroot.sh,v 0.2 2002/07/03 15:34:20 john Exp $
